#!/usr/bin/env ruby
# MoP - Mozilla de Presentation
# HTML source splitter
# (C)2005, 2007 by HIROSE, Yuuji [yuuji@yatex.org]
# $Id: mop.rb,v 1.9 2007/02/16 22:55:25 yuuji Exp $
# Last modified Sat Feb 17 07:54:12 2007 on firestorm
# Update count: 67

$ropt = Regexp::MULTILINE|Regexp::IGNORECASE
$KCODE = 'e'
$base="slide-%02d.html"
$mag=4
$event = 'OnMouseOver';

$count=$subcount=0

def chasemydir()
  target = $0
  limit = 30
  pwd=Dir.pwd
  begin
    Dir.chdir(File.dirname(target))
    while test(?l, target) && (limit-=1) > 0
      target = File.readlink(target)
      Dir.chdir(File.dirname(target))
      target = File.basename(target)
    end
    return Dir.pwd
  ensure
    Dir.chdir pwd
  end
end

mydir=chasemydir
$tex2png="perl -I #{mydir}/ps2img #{mydir}/tex2png"

def getcontent(content, element)
  regx = sprintf("<%s[^>]*>\\s*(.*)\\s*</%s\\b", element, element)
  content.scan(Regexp.new(regx, $ropt))
end
def tagify(elm, attr, content, cr=nil)
  nl=(cr ? "\n" : "")
  x = attr.collect do |attr, val|
    sprintf("%s=\"%s\"", attr, val)
  end.join(" ")
  sprintf("<%s%s>%s%s%s</%s>", elm, (x>"" ? " "+x : ""), nl, content, nl, elm)
end

def href(base, n, max)
  left=nil
  right=nil
  if n>1
    left=tagify("a", {"href"=>sprintf(base, n-1), "id"=>"_prev"}, "&lt;=PREV")
  end
  if n<max
    right=tagify("a", {"href"=>sprintf(base, n+1), "id"=>"_next"}, "NEXT=&gt;")
  end
  links = [left, right]
  links.delete(nil)
  sprintf("<div class=\"c pre small\" id=\"lrlink\">\n%s\n</div>",
	  links.join(" | "))
end

def escape(string)
  string.gsub(/([\"'<>{}])/n) do
      '&#x' + $1.unpack('H2').join('').upcase + ';'
  end
end

def getattr(attr, str)
  str && /#{attr}=([\"\'])(\w+)\1/ =~ str && $2
end

def latex(text, args, tex2png=$tex2png)
  img = sprintf($base, $count)
  img.sub!(/\.html/, sprintf("-%02d.png", $subcount+=1))
  mag = (/mag=([\"\'])([\d.]+)\1/ =~ args ? \
	 $2 : $mag.to_s)
  cmd = sprintf("%s '%s' %s %s", tex2png, text, img, mag)
  STDERR.printf("CMD=%s\n",cmd) if $DEBUG
  system(cmd)
  sprintf("<img src=\"%s\" alt=\"%s\">", img, escape(text))
end

def eqn(text, args)
  latex(text, args, $tex2png.sub(/tex/, "eqn"))
end

def filter(text)
  # Feed contents in div to filter(s)
  filters=/<(div|span) class=([\"']?)(latex|eqn|tag|emerge|opaque|rcutin|dialog|qanda)\2([^>]*)>/

  while filters =~ text
    tag   = $1
    left  = $`
    # right = $'
    cmd   = $3
    args  = $4

    beg  = $~.end(0)
    while /\s/ =~ text[beg, 1];  beg += 1; end
    nest = 0
    rest = text[beg..-1]
    inner = ""
#printf "beg=%d, rest={%s}\n", beg, rest
    delim=%r,(<#{tag}\b.*?>)|(</#{tag}>),
    event = getattr("event", args) || $event
    nl = (/div/i =~ tag ? "\n" : "")
    while delim =~ rest
      inner << $`
      rest = $'
      if $~.begin(1)		# if new <div> starts
	nest += 1
      else			# /div
	nest -= 1
	break if nest < 0
      end
      inner << $&
    end
    target = inner.chomp("\n")
    case cmd
    when "latex"
      text = left + latex(filter(target), args) + rest
    when "eqn"
      text = left + eqn(filter(target), args) + rest
    when "tag"
      text = left + "<<" + filter(target) + ">>" + rest
    when "emerge", "opaque", "rcutin"
      id="id#{$idnum+=1}"
      text = left +
	"<#{tag} id=\"#{id}p\" class=\"#{cmd}p\" "+
	"#{event}=\"#{cmd}('#{id}')\">#{nl}" +
	"<#{tag} id=\"#{id}\" class=\"emerge\">#{nl}" +
	filter(target) +
	"</#{tag}>#{nl}</#{tag}>#{nl}" + rest
#     when "rcuin"
#       id="id#{$idnum+=1}"
#       text = left +
# 	"<#{tag} id=\"#{id}\" class=\"#{cmd}\" " +
# 	"#{event}=\"#{cmd}('#{id}')\">#{nl}" +
# 	filter(target) +
# 	"</#{tag}>#{nl}" + rest
    when "dialog", "qanda"
      id="id#{$idnum+=1}"
      text = left +
	"<#{tag} id=\"#{id}\" class=\"#{cmd}\" " +
	"onMouseDown=\"d_down(event)\">#{nl}" +
	filter(target) +
	"</#{tag}>#{nl}" + rest
    end
#    p text
  end
  text
end

def init(dir, cmd="ln -s")
  mydir=chasemydir
  for file in %w(basic.css mop.css mop.js q.png ans.png don.png taro-l.png
		 hanako-r.png)
    f = File.join(dir, file)
    test(?r, f) || system("#{cmd} #{File.join(mydir, file)} #{f}")
  end
  for file in %(template.html)
    system("cp #{File.join(mydir, file)} #{dir}")
  end
  exit 0
end

def usage()
  me = File.basename($0)
  puts <<_EOU_
Usage:
	#{me} -init		Start from this!
				Create Symlink of neccessary files
				ɬץե뷲symlinkǺ
	#{me} -cp		Like -init, but copy files
				-initƱȤcpǤ
	#{me} file.html	Split file.html up for presentation
				file.htmlץ쥼󥷡ȷʬ

Steps for making presentation Web by #{me}.
 1. Create empty directory for presentation and chdir there.
 2. Call \`mop.rb -init' to prepare files.
 3. Rename or copy template.html to your own HTML file (eg. pr.html).
 4. Edit it and call \`mop.rb pr.html'
 5. See slide-01.html with mozilla/firefox! (Safari/Opera/IE also welcome)

See also
http://www.gentei.org/~yuuji/software/mop/
ߤƤ͡

#{me}ǥץ쥼󥷡Ⱥ
 1. ץ쥼ѤζΥǥ쥯ȥäƤ˰ư
 2. mop.rb -initפ롣ե뤬Ǥ롣
 3. template.html 򹥤̾HTML˥ԡ͡(Ȥpr.html)
 4. pr.htmlԽ mop.rb pr.htmlפ롣
 5. 夬ä slide-01.html mozilla/firefoxǸ롣

_EOU_
  exit 0
end

##
# Parse Command Line Options
##
splitby = "h1"
hshift=0
outdir="."
while ARGV[0]
  break unless /^-/ =~ ARGV[0]
  $_ = ARGV[0].dup
  while ~/^-(.)/
    case $_
    when /-h/
      usage()
    when /-o/
      outdir=ARGV.shift
    when /^-cp/
      init(outdir, "cp")
    when /^-init/
      init(outdir)
    when /^-([123456])/
      splitby = "h"+$1
      hshift = $1.to_i - 1
    end
    sub!(/^-(.)(.*)/, "-"+'\2')
  end
  ARGV.shift
end

all=readlines.join("")
header = all.split(/<body\b/)[0]
title = getcontent(header, "title")
bbody = all.scan(/<body[^>]*>/)[0]

body=getcontent(all, "body")[0][0]

if hshift > 0
  ptn = "h([#{hshift+1}-6])"
  body.gsub!(/\b#{ptn}>/){
    "h" + ($1.to_i - hshift).to_s + ">"
  }
end

block=body.split(/(<h1[^>]*>.*?<\/h1>)/m)
block.shift

total=block.length/2
while h1=block.shift
  rest=block.shift
  $subcount=0
  fn=sprintf($base, $count+=1)
  newtitle = sprintf("<title>%d/%d - %s</title>", $count, total, title)
  newheader = header.sub(/<title[^>]*>.*<\/title>/m, newtitle)
  STDERR.print '['
  $idnum = 0;
  open(fn, "w") do |slide|
    slide.print newheader
    if $count==1
      slide.puts "<body class=\"titlepage\">"
    else
      slide.puts bbody.sub(/>/, " onLoad=\"init(#{total});\">")
    end
    slide.print h1
    bottom = tagify("div", {"id"=>"_bottom"}, ".", false);
    hr = href($base, $count, total)
    s1 = tagify("div", {"class" => "s1"}, filter(rest), true)
    slide.puts tagify("div", {"class" => "s0"}, s1+hr+bottom, true)
#slide.puts hr+bottom;
    slide.puts "</body>\n"
    slide.puts "</html>\n"
  end
  STDERR.printf '%d]', $count
end
STDERR.puts ''

while test(?r, fn=sprintf($base, $count+=1))
  File.unlink(fn)
end
